//-----------------------------------------------------------------------------
// USART Fuction Declaration

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
// UART Defination
//------------------------------------------------------------------------------
#define BAUD_9600							0
#define BAUD_14400							1
#define BAUD_19200							2
#define BAUD_28800							3
#define BAUD_38400							4		
#define BAUD_57600							5		
#define BAUD_76800							6		
#define BAUD_115200							7	

#ifdef _SYS_CLK_AT8MHZ
#define BAUDRATE_VAL_9600					207
#define BAUDRATE_VAL_19200					0x36	//at. 1Mhz
#define BAUDRATE_VAL_14400					138
#define BAUDRATE_VAL_28800					68
#define BAUDRATE_VAL_38400					208
#define BAUDRATE_VAL_57600					139		//at. 8Mhz					
#define BAUDRATE_VAL_76800					25
#define BAUDRATE_VAL_115200					64		//8Mhz/115200 - 69.44
#endif

#ifdef _SYS_CLK_AT1MHZ
#define BAUDRATE_VAL_9600					207
#define BAUDRATE_VAL_19200					0x36	//at. 1Mhz
#define BAUDRATE_VAL_14400					138
#define BAUDRATE_VAL_28800					68
#define BAUDRATE_VAL_38400					51
#define BAUDRATE_VAL_57600					18		//at. 1Mhz					
#define BAUDRATE_VAL_76800					25
#define BAUDRATE_VAL_115200					64		//8Mhz/115200 - 69.44
#endif

#define BAUDRATE_VAL_DEFAULT				BAUDRATE_VAL_57600

#define UART_DB_BAUDRATE					0x02

#define USART_BAUD_SELECT (((F_CPU / 100) / (16 * (USART_BAUD_RATE / 100))) - 1)
#define USART_BAUD_L (USART_BAUD_SELECT & 0x00ff)
#define USART_BAUD_H ((USART_BAUD_SELECT & 0xff00) >> 8)

#define UART_RXINT_ENABLE           0x80
#define UART_TXINT_ENABLE           0x40
#define UART_RX_ENABLE              0x10
#define UART_TX_ENABLE              0x08

#define UART_RX_COMPLETE            0x80
#define UART_TX_COMPLETE            0x40
#define UART_DATA_EMPTY             0x20

#define TXD0_SEL_EN						0x10
#define RXD0_SEL_EN						0x20
#define TXD1_SEL_EN						0x40
#define RXD1_SEL_EN						0x80

#ifdef _UART0_ENABLE
//------------------------------------------------------------------------------
// Serial Prot0 Utility Fuction Routine
#define u0_WaitForReceive()			while(!(IFG1 & URXIFG0))
#define u0_RxData()		 			(RXBUF0)

#define _u0_TxData(Data)				(TXBUF0 = (Data))
#define u0_WaitForSend()			while (!(IFG1 & UTXIFG0)) 

#define u0_IsReceiveData()			(IFG1 & URXIFG0)

void u0_baudrate(unsigned char baud);
void u0_hal_init(void);

void u0_TxData(unsigned char Data)
{
	_u0_TxData(Data);
}

void u0_hal_init(void)
{
	P3SEL |= TXD0_SEL_EN+RXD0_SEL_EN;
  	ME1 |= UTXE0 + URXE0;                 // Enabled USART0 TXD/RXD	
	UCTL0 |= CHAR;                        // 8-bit character, SWRST = 1
	UTCTL0 |= SSEL1 + SSEL0 + URXSE;      // UCLK = SMCLK, start edge detect
	
	UMCTL0 = 0x00;                            // no modulation
//  	UMCTL0 = 0x44;                        // Modulation = .25	
	UCTL0 &= ~SWRST;                      // Initialize USART state machine(software reset)
}

void u0_baudrate(unsigned char baud)
{
	switch (baud)
	{
		case BAUD_9600:
			
			break;
		case BAUD_14400:
			
			break;		
		case BAUD_19200:
			UBR00 = BAUDRATE_VAL_19200&0xFF;
			UBR10 = (BAUDRATE_VAL_19200>>8)&0xFF;
			break;		
		case BAUD_28800:
			
			break;		
		case BAUD_38400:
			
			break;		
			
		case BAUD_57600:
			UBR00 = BAUDRATE_VAL_57600&0xFF;
			UBR10 = (BAUDRATE_VAL_57600>>8)&0xFF;
			break;
			
		case BAUD_76800:
			
			break;		
		case BAUD_115200:
			UBR00 = BAUDRATE_VAL_115200&0xFF;
			UBR10 = (BAUDRATE_VAL_115200>>8)&0xFF;			
			break;		
		default :
						
			break;
	}	
	
	
}

int U0_printf(const char *format, ...)
{
    int i = 0;
    
    va_list ap; 
    char string[40]; //<--- ڿ ִ 

    va_start(ap,format); 
    vsprintf(string,format,ap); 

    for(i=0;i<40;i++)
    {
        if(string[i]==0)break;
        u0_WaitForSend();
        _u0_TxData(string[i]);
    }

    va_end(ap);

    return i; 
}
#endif
//------------------------------------------------------------------------------
