//=============================================================================
// msp430 HAL
// by nexp76
// [system.c]
//=============================================================================

/*
 - [2007.01.02] 
 - UART0 HAL 
 - printf -> U0_printf  
*/


#include "system.h"

#ifdef _SYS_CLK_AT1MHZ
void Set_DCO (void); 
#endif

void Delay(unsigned int Time)
{
    int i, j;
    for(i=0;i<Time;i++)
	   	for(j=0;j<200;j++);
}

void OscCalibation(void)
{
    // CLK - 8Mhz Calibration ־   Ѵ.
    int i = 0;
    
    do
    {
        IFG1 &= ~OFIFG;                           // Clear OSCFault flag
        for (i = 0xFF; i > 0; i--);               // Time for flag to set
    }
    while ((IFG1 & OFIFG));                   // OSCFault flag still set?
}
  
void SystemInit(void)
{
	WDTCTL = WDTPW + WDTHOLD;             // Stop WDT
	
	#ifdef _SYS_CLK_AT8MHZ
	BCSCTL1 &= ~XT2OFF;                       // XT2on
	
	OscCalibation();
	
	BCSCTL2 |= SELM_2 + SELS;                 // MCLK= SMCLK= XT2 (safe)	
	#endif
	
	//BCSCTL1 |= DIVA_3;                    // ACLK= LFXT1CLK/8
	//Set_DCO();                            // Set DCO	
}

#ifdef _SYS_CLK_AT1MHZ
//------------------------------------------------------------------------------ 
void Set_DCO (void)                     // Set DCO to selected frequency
//------------------------------------------------------------------------------ 
{
#define DELTA 256                       // Target DCO = DELTA*(4096) = 1048576  

  unsigned int Compare, Oldcapture = 0;

  CCTL2 = CM_1 + CCIS_1 + CAP;          // CAP, ACLK
  TACTL = TASSEL_2 + MC_2 + TACLR;      // SMCLK, cont-mode, clear

  while (1)
  {
    while (!(CCIFG & CCTL2));           // Wait until capture occured
    CCTL2 &= ~CCIFG;                    // Capture occured, clear flag
    Compare = CCR2;                     // Get current captured SMCLK 
    Compare = Compare - Oldcapture;     // SMCLK difference
    Oldcapture = CCR2;                  // Save current captured SMCLK
    if (DELTA == Compare) break;        // If equal, leave "while(1)"

    else if (DELTA < Compare)           
    {
      DCOCTL--;
      if (DCOCTL == 0xFF)               // DCO is too fast, slow it down
      {
        if (!(BCSCTL1 == (XT2OFF + DIVA_3))) 
        BCSCTL1--;                      // Did DCO role under?, Sel lower RSEL
      }
    }
    else
    {                      
      DCOCTL++;                         // DCO is too slow, speed it down
      if (DCOCTL == 0x00) 
      {
        if (!(BCSCTL1 == (XT2OFF + DIVA_3 + 0x07))) 
        BCSCTL1++;                      // Did DCO role over? Sel higher RSEL
      }
    }
  }

  CCTL2 = 0;                            // Stop CCR2
  TACTL = 0;                            // Stop Timer_A
}
#endif
//------------------------------------------------------------------------------ 
	